@@ -3,6 +3,8 @@
 # pylint: disable=missing-function-docstring
 
 
+import pytest
+
 from quantify_core.data import handling as dh
 from quantify_core.data.experiment import QuantifyExperiment
 
@@ -37,3 +39,16 @@ def test_quantify_experiment_load_and_save_text(tmp_test_data_dir):
     experiment.save_text(text, rel_path)
 
     assert experiment.load_text(rel_path) == text
+
+
+def test_quantify_experiment_load_and_save_metadata(tmp_test_data_dir):
+    dh.set_datadir(tmp_test_data_dir)
+    tuid = TUID_1D_1PLOT
+    dh.create_exp_folder(tuid)
+    experiment = QuantifyExperiment(tuid)
+    with pytest.raises(FileNotFoundError):
+        _ = experiment.load_metadata()
+
+    dictionary = {"key": "entry to be saved"}
+    experiment.save_metadata(dictionary)
+    assert experiment.load_metadata() == dictionary
